import { MatchRequest } from '../../../../domain/dtos/match.request';
import { Candidate } from '../../../../domain/entities/engine/candidate';
import { AlgorithmFactory } from '../../../../domain/entities/engine/factory/algorithm-factory.abstract';
import { Processor } from '../../../../domain/entities/engine/processor/processor.abstract';
import { Selector } from '../../../../domain/entities/engine/selector/selector.abstract';
import { AlgorithmType } from '../../../../domain/types/algorithm.enum';
import { IDefaultParams } from '../../../../domain/types/default-params.type';
import { MatchQuery } from '../../../../queries/match.query';

const mockGeorouterCreator = {
  create: jest.fn().mockImplementation(),
};

const mockTimezoneFinder = {
  timezones: jest.fn().mockImplementation(),
};

const defaultParams: IDefaultParams = {
  DEFAULT_IDENTIFIER: 0,
  MARGIN_DURATION: 900,
  VALIDITY_DURATION: 365,
  DEFAULT_TIMEZONE: 'Europe/Paris',
  DEFAULT_SEATS: 3,
  DEFAULT_ALGORITHM_SETTINGS: {
    algorithm: AlgorithmType.CLASSIC,
    strict: false,
    remoteness: 15000,
    useProportion: true,
    proportion: 0.3,
    useAzimuth: true,
    azimuthMargin: 10,
    maxDetourDistanceRatio: 0.3,
    maxDetourDurationRatio: 0.3,
    georouterType: 'graphhopper',
    georouterUrl: 'http://localhost',
  },
};

const matchRequest: MatchRequest = new MatchRequest();
matchRequest.departure = '2023-04-01 12:00';
matchRequest.waypoints = [
  {
    lat: 49.440041,
    lon: 1.093912,
  },
  {
    lat: 50.630992,
    lon: 3.045432,
  },
];
const matchQuery: MatchQuery = new MatchQuery(
  matchRequest,
  defaultParams,
  mockGeorouterCreator,
  mockTimezoneFinder,
);

class FakeSelector extends Selector {
  select = (): Promise<Candidate[]> => {
    return Promise.resolve([new Candidate()]);
  };
}

class FakeProcessor extends Processor {
  execute = (candidates: Candidate[]): Candidate[] => {
    return candidates;
  };
}

class FakeAlgorithmFactory extends AlgorithmFactory {
  createSelector = (): Selector => {
    return new FakeSelector(matchQuery);
  };
  createProcessors = (): Processor[] => {
    return [new FakeProcessor(matchQuery)];
  };
}

describe('AlgorithmFactory', () => {
  it('should create an extended class', () => {
    expect(new FakeAlgorithmFactory(matchQuery)).toBeDefined();
  });
});
